﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Diagnostics;
using System.Threading;

namespace Oracle.VirtualBox.Tests
{
    /// <summary>
    /// Summary description for UnitTests
    /// </summary>
    [TestClass]
    public class UnitTests
    {

        [TestMethod]
        public void CreateProcess_OnWindows8_Succeeds()
        {
            //add web service reference to VirtualBox
            //add reference to System.Web.Services

            //running on Windows 7 64-bit
            //Oracle VM VirtualBox 4.2.6r82870

            //"C:\Program Files\Oracle\VirtualBox\VBoxManage.exe" setproperty websrvauthlibrary null

            //run VirtualBox non-elevated

            //run the following command in a non-elevated command prompt:
            //"c:\Program Files\Oracle\VirtualBox\VBoxWebSrv.exe" --verbose

            vboxBinding webServiceClient = new vboxBinding(new Uri("http://localhost:18083/"));

            //Windows 8 32-bit
            string virtualMachineNameOrId = "Win8-PC";

            //Base snapshot was created when specified user is logged in and after guest additions are installed
            //VirtualBox Guest Additions 4.2.6r82870
            string snapshotNameOrId = "Base";

            string username = "rami.abughazaleh@live.com";
            string password = "password";

            string filePath = @"c:\windows\system32\notepad.exe";
            string webSessionId = null;
            string sessionId = null;

            try
            {
                //log into service and get session id
                webSessionId = webServiceClient.IWebsessionManager_logon("", "");
                sessionId = webServiceClient.IWebsessionManager_getSessionObject(webSessionId);

                //get version
                string version = webServiceClient.IVirtualBox_getVersion(webSessionId);

                Trace.WriteLine(version);
                
                //get virtual machine
                string virtualMachineId = webServiceClient.IVirtualBox_findMachine(webSessionId, virtualMachineNameOrId);

                //power off virtual machine
                webServiceClient.IMachine_lockMachine(virtualMachineId, sessionId, LockType.Shared);

                string consoleId = webServiceClient.ISession_getConsole(sessionId);

                MachineState machineState = webServiceClient.IConsole_getState(consoleId);

                if (!((machineState == MachineState.Saved)
                    || (machineState == MachineState.PoweredOff)))
                {
                    string progressId = webServiceClient.IConsole_powerDown(consoleId);

                    webServiceClient.IProgress_waitForCompletion(progressId, -1);

                    string errorInfo = webServiceClient.IProgress_getErrorInfo(progressId);
                    if (!string.IsNullOrEmpty(errorInfo))
                    {
                        string errorText = webServiceClient.IVirtualBoxErrorInfo_getText(errorInfo);

                        throw new Exception(string.Format("{0} {1}", errorText, errorInfo));
                    }
                }

                SessionState sessionState = webServiceClient.ISession_getState(sessionId);

                if (sessionState == SessionState.Locked)
                {
                    webServiceClient.ISession_unlockMachine(sessionId);
                }

                //restore snapshot
                {
                    webServiceClient.IMachine_lockMachine(virtualMachineId, sessionId, LockType.Shared);

                    consoleId = webServiceClient.ISession_getConsole(sessionId);

                    string snapshotId = webServiceClient.IMachine_findSnapshot(virtualMachineId, snapshotNameOrId);

                    string progressId = webServiceClient.IConsole_restoreSnapshot(consoleId, snapshotId);

                    webServiceClient.IProgress_waitForCompletion(progressId, -1);

                    string errorInfo = webServiceClient.IProgress_getErrorInfo(progressId);
                    if (!string.IsNullOrEmpty(errorInfo))
                    {
                        string errorText = webServiceClient.IVirtualBoxErrorInfo_getText(errorInfo);

                        throw new Exception(string.Format("{0} {1}", errorText, errorInfo));
                    }

                    sessionState = webServiceClient.ISession_getState(sessionId);

                    if (sessionState == SessionState.Locked)
                    {
                        webServiceClient.ISession_unlockMachine(sessionId);
                    }
                }

                //power on virtual machine
                {
                    string progressId = webServiceClient.IMachine_launchVMProcess(virtualMachineId, sessionId, "gui", "");

                    webServiceClient.IProgress_waitForCompletion(progressId, -1);

                    string errorInfo = webServiceClient.IProgress_getErrorInfo(progressId);
                    if (!string.IsNullOrEmpty(errorInfo))
                    {
                        string errorText = webServiceClient.IVirtualBoxErrorInfo_getText(errorInfo);

                        throw new Exception(string.Format("{0} {1}", errorText, errorInfo));
                    }
                }

                //wait for guest additions
                consoleId = webServiceClient.ISession_getConsole(sessionId);

                string guestId = webServiceClient.IConsole_getGuest(consoleId);

                string osTypeId = webServiceClient.IGuest_getOSTypeId(guestId);

                Trace.WriteLine(osTypeId);

                AdditionsRunLevelType additionsRunLevelType = webServiceClient.IGuest_getAdditionsRunLevel(guestId);

                if (additionsRunLevelType == AdditionsRunLevelType.None)
                {
                    throw new Exception("Guest additions not installed");
                }

                Trace.WriteLine(additionsRunLevelType);

                bool active = webServiceClient.IGuest_getAdditionsStatus(guestId, AdditionsRunLevelType.System);

                while (!active)
                {
                    Thread.Sleep(3000);

                    active = webServiceClient.IGuest_getAdditionsStatus(guestId, AdditionsRunLevelType.System);
                }

                Trace.WriteLine("Guest drivers are loaded.");

                if (additionsRunLevelType == AdditionsRunLevelType.System)
                {
                    return;
                }

                active = webServiceClient.IGuest_getAdditionsStatus(guestId, AdditionsRunLevelType.Userland);

                while (!active)
                {
                    Thread.Sleep(3000);

                    active = webServiceClient.IGuest_getAdditionsStatus(guestId, AdditionsRunLevelType.Userland);
                }

                Trace.WriteLine("Common components (such as application services) are loaded.");

                if (additionsRunLevelType == AdditionsRunLevelType.Userland)
                {
                    return;
                }

                active = webServiceClient.IGuest_getAdditionsStatus(guestId, AdditionsRunLevelType.Desktop);

                while (!active)
                {
                    Thread.Sleep(3000);

                    active = webServiceClient.IGuest_getAdditionsStatus(guestId, AdditionsRunLevelType.Desktop);
                }

                Trace.WriteLine("Per-user desktop components are loaded.");

                //log into guest and check if file exists
                string guestSessionId = webServiceClient.IGuest_createSession(guestId, username, password, null, "sessionName");

                webServiceClient.IGuest_setCredentials(guestId, username, password, null, true);

                Trace.WriteLine(string.Join(Environment.NewLine, webServiceClient.IGuestSession_getId(guestSessionId)));
                Trace.WriteLine(string.Join(Environment.NewLine, webServiceClient.IGuestSession_getDomain(guestSessionId)));
                Trace.WriteLine(string.Join(Environment.NewLine, webServiceClient.IGuestSession_getDirectories(guestSessionId)));
                Trace.WriteLine(string.Join(Environment.NewLine, webServiceClient.IGuestSession_getFiles(guestSessionId)));
                Trace.WriteLine(string.Join(Environment.NewLine, webServiceClient.IGuestSession_getProcesses(guestSessionId)));
                Trace.WriteLine(string.Join(Environment.NewLine, webServiceClient.IGuestSession_getEnvironment(guestSessionId)));
                Trace.WriteLine(string.Join(Environment.NewLine, webServiceClient.IGuestSession_getName(guestSessionId)));
                Trace.WriteLine(string.Join(Environment.NewLine, webServiceClient.IGuestSession_getUser(guestSessionId)));

                //System.Web.Services.Protocols.SoapException: VirtualBox error: Querying file information for "c:\windows\system32\notepad.exe" failed: VERR_NOT_FOUND (0x80BB0005)
                bool fileExists = webServiceClient.IGuestSession_fileExists(guestSessionId, filePath);

                Trace.WriteLine(fileExists);
            }
            finally
            {
                //unlock machine
                if (sessionId != null)
                {
                    SessionState sessionState = webServiceClient.ISession_getState(sessionId);

                    if (sessionState == SessionState.Locked)
                    {
                        webServiceClient.ISession_unlockMachine(sessionId);
                    }
                }

                //log off
                if (webSessionId != null)
                {
                    Trace.WriteLine("logging off");

                    webServiceClient.IWebsessionManager_logoff(webSessionId);
                }
            }
        }
    }
}
